@@ -159,7 +159,7 @@ void ConstructEqn (PINPUTBUF pibIn, PSTR szRName, SpeciesReference_t *species,
 
   sprintf (szStoichio, "%g", SpeciesReference_getStoichiometry (species));  
 
-  if (szStoichio == "0") {
+  if (!MyStrcmp(szStoichio, "0")) {
     sprintf (szStoichio, "1");
     printf (" %s stoichio was 0, now set to %s\n", szSName, szStoichio);
   }
@@ -752,7 +752,6 @@ void ReadReaction_L1 (PINPUTBUF pibIn, ListOf_t *reaction_list, long index)
   /* define reaction name as a local variable in the Dynamics section */ 
   DefineVariable (pibIn, szRName, szEqn, 0);
 
-
 } /* ReadReaction_L1 */
 
 
@@ -793,7 +792,7 @@ void ReadReaction_L2 (PINPUTBUF pibIn, ListOf_t *reaction_list, long index)
 
   /* get local parameter list first */
   param_list = KineticLaw_getListOfParameters(kl);
-  p = ListOf_size  (param_list);
+  p = ListOf_size (param_list);
 
   printf (" number of local parameters (made global by MCSim): %ld\n", p);
 
@@ -987,9 +986,40 @@ void Read1Species (PINPUTBUF pibIn, BOOL bProcessPK_ODEs, ListOf_t *species_list
   Species_t *species;
 
   species = (Species_t *) ListOf_get (species_list, index);
-  sprintf (szID, "%s", Species_getId (species));
-  sprintf (szEqn,  "%g", Species_getInitialConcentration (species));
-  sprintf (szCpt,  "%s", Species_getCompartment (species));
+
+  switch (iIDtype) {
+    case ID:
+      sprintf (szID,  "%s", Species_getId (species));
+      break;
+    case name:
+      sprintf (szID,  "%s", Species_getName (species));
+      break;
+    case metaID:
+      sprintf (szID,  "%s", SBase_getMetaId ((SBase_t *) species));
+      break;
+  }
+
+  if (Species_getHasOnlySubstanceUnits (species)) {
+    if (Species_isSetInitialAmount (species))
+      sprintf (szEqn, "%g", Species_getInitialAmount (species));
+    else {
+      printf ("***Error: Species has only substance units but "
+              "InitialAmount is not set.\n"
+              "Exiting.\n\n");
+      exit (0);
+    }
+  }
+  else {
+    if (Species_isSetInitialConcentration (species))
+      sprintf (szEqn, "%g", Species_getInitialConcentration (species));
+    else {
+      sprintf (szEqn, "%g", Species_getInitialAmount (species));
+      printf ("\nWarning: Species should be concentration but "
+              "InitialConcentration is not set.\n\n");
+    }
+  }
+
+  sprintf (szCpt, "%s", Species_getCompartment (species));
   bBoundary = (BOOL) Species_getBoundaryCondition (species);
 
   if (!strcmp (szID, "(null)")) {
@@ -1031,6 +1061,7 @@ void Read1Species (PINPUTBUF pibIn, BOOL bProcessPK_ODEs, ListOf_t *species_list
       else /* not boundary, create a state variable */
         SetVar (pibIn, szID, szEqn, ID_STATE);
     } /* if */
+
     else { /* species is outside of a template-defined compartment */
 
       printf (" (circulating species)\n");
@@ -1191,31 +1222,28 @@ void ReadOptions (PINPUTBUF pibIn)
      use defaults */
   SkipWhitespace (pibIn);
   if (*pibIn->pbufCur == '"') { 
-    iIDtype = ID; /* practically mandatory because IDs are used in reactions */
-    strcpy (szDefault_Cpt, "default"); /* works for CellDesigner. JDesigner wants
-                                          "compartment" */
+    iIDtype = ID; /* default because IDs are used in reactions */
+    strcpy (szDefault_Cpt, "default");
   }
   else  {
 
     /* get UseName, UseID or UseMetaID option */
     NextLex (pibIn, szLex, &iLexType);
 
-    { /* switch according to szLex */
-     if (!strcmp (szLex, "UseID"))
-       {iIDtype = ID;     goto Done;}
-#ifdef ndef
-     /* disabled for now */
-     if (!strcmp (szLex, "UseName"))   /* would be problematic for reactions */
-       {iIDtype = name;   goto Done;}
-     if (!strcmp (szLex, "UseMetaID")) /* would be problematic for reactions */
-       {iIDtype = metaID; goto Done;}
-#endif
-     /* if we arrive here there is a problem */
-     printf ("***Error: SBMLModel 1st option must be 'UseID'.\n"
-             "'UseName' and 'UseMetaID' are disabled for now. "
-             "Exiting.\n\n");
-     exit(0);
-    } Done: /* Label... */
+    /* switch according to szLex */
+    if (!strcmp (szLex, "UseID"))
+      {iIDtype = ID;     goto Done;}
+    if (!strcmp (szLex, "UseName"))   /* is problematic for reactions */
+      {iIDtype = name;   /* goto Done; */}
+    if (!strcmp (szLex, "UseMetaID")) /* is problematic for reactions */
+      {iIDtype = metaID; /* goto Done; */}
+    /* if we arrive here there is a problem */
+    printf ("***Error: SBMLModel 1st option must be 'UseID'." 
+                       "'UseName' or 'UseMetaID' are disabled for now.\n"
+            "Exiting.\n\n");
+    exit(0);
+
+    Done:
 
     NextLex (pibIn, szPunct, &iLexType);
     SkipWhitespace (pibIn);
@@ -1263,24 +1291,24 @@ void ReadSBMLModels (PINPUTBUF pibIn)
     /* init buffer and read in the input file. */
     /* buffer size -1 will create a buffer of the size of the input file */ 
     if (!InitBuffer (&ibInLocal, -1, pszFileNames[i]))
-      ReportError (&ibInLocal, RE_INIT | RE_FATAL, "ReadJModels", NULL);
+      ReportError (&ibInLocal, RE_INIT | RE_FATAL, "ReadSBMLModels", NULL);
 
     /* attach info records to input buffer */
     ibInLocal.pInfo = pibIn->pInfo;
     ibInLocal.pTempInfo = pibIn->pTempInfo;
 
-    document = readSBML(pszFileNames[i]);
+    document = readSBML (pszFileNames[i]);
 
-    SBMLDocument_printErrors(document, stdout);
+    SBMLDocument_printErrors (document, stdout);
 
-    model = SBMLDocument_getModel(document);
+    model = SBMLDocument_getModel (document);
     if (model == NULL) {
       printf ("***Error: No model present. Exiting.\n\n");
       exit (0);
     }
 
     /* read the SBML level */
-    iSBML_level = ReadSBMLLevel(document);
+    iSBML_level = ReadSBMLLevel (document);
 
     /* PK template requires level 2 SBML, issue an error otherwise */
     if ((pinfo->bTemplateInUse) && (iSBML_level < 2)) {
@@ -1328,7 +1356,7 @@ void ReadSBMLModels (PINPUTBUF pibIn)
 
     /* reinitialize model */
     document = readSBML(pszFileNames[i]);
-    model = SBMLDocument_getModel(document);
+    model = SBMLDocument_getModel (document);
 
     /* re-read compartments, no printing, no redeclaration */
     ReadCpts (&ibInLocal, model, FALSE);
