@@ -139,83 +139,16 @@ void CheckPrintStatements (PLEVEL plevel, char **args)
         ReportRunTimeError (panal, RE_DUPVARINEXPRT | RE_FATAL,
                             pos->pszDataNames[i], "Data");
 
-#ifdef ndef
-
-  /* this section needs to be completely revamped */
-
-  /* check that all Predictions or Data appearing in Distrib statements have
-     a corresponding output or data specification and that these have same
-     output times */
-  for (i = 0; i < plevel->nMCVars; i++) {
-    pMCVar = plevel->rgpMCVars[i]; /* current Distrib examined */
-    if (!IsParm (pMCVar->hParm[0])) { /* deal with first parameter */
-        /* it's a state, input or output, it must have had a Prediction
-           or a Data keyword and must have an output or data */
-        bFound = FALSE;
-        /* check the outputs */
-        for (k = 0; (k < pos->nOutputs) || bFound; k++) {
-          if (bFound = (pMCVar->hParm[0] == pos->phvar_out[k])) {
-            dCount = pos->pcOutputTimes[k]; /* init dCount */
-            dIndex = k;
-          }
-        }
-        if (!bFound) {
-          /* check the data statements */
-          for (k = 0; (k < pos->nData) || bFound; k++) {
-            if (bFound = (pMCVar->hParm[0] == pos->phvar_dat[k])) {
-              dCount = pos->pcData[k]; /* init dCount */
-              dIndex = k;
-            }
-          }
-        }
-
-        if (!bFound) /* still not found */
-          ReportRunTimeError (panal, RE_NOPDSTATEMENT | RE_FATAL,
-                              pMCVar->pszName);
+  /* check that print and matching data lists are of equal length */
+  for (j = 0; j < pos->nOutputs; j++)
+    for (i = 0; i < pos->nData; i++)
+      if ((pos->phvar_out[j] == pos->phvar_dat[i]) &&
+          (pos->pcOutputTimes[j] != pos->pcData[i])) {
+        printf("\nError: unequal times in Print and Data statements for %s\n"
+               "Exiting.\n\n", pos->pszOutputNames[j]);
+        exit(0);
       }
 
-    /* now check that distributional parameters which are Data or Prediction
-       have data or print statements and adequate number of output times or
-       data items */
-    for (j = 1; j < 4; j++) {
-      if (!IsParm (pMCVar->hParm[j])) {
-        bFound = FALSE;
-        for (k = 0; (k < pos->nOutputs) || bFound; k++) {
-          if (bFound = (pMCVar->hParm[j] == pos->phvar_out[k])) {
-            bOK = (dCount == pos->pcOutputTimes[k]);
-            for (l = 0; (l < dCount) && bOK; l++)
-              bOK = (pos->prgdOutputTimes[dIndex][l] ==
-                     pos->prgdOutputTimes[k][l]);
-          }
-          if (!bOK)
-            ReportRunTimeError (panal, RE_UNEQUALTIMES | RE_FATAL,
-                                GetVarName (pMCVar->hParm[0]),
-                                GetVarName (pMCVar->hParm[j]),
-                                pMCVar->pszName);
-        } /* for k */
-
-        if (!bFound)
-          ReportRunTimeError (panal, RE_NOPRINTSTATEMENT | RE_FATAL,
-                              pMCVar->pszName);
-      }
-    } /* for j */
-  } /* for i */
-
-  /* check that each variable for which Data are specified also have a
-     Distrib statement */
-  for (i = 0; i < pos->nData; i++) {
-    bFound = FALSE;
-    for (j = 0; (j < plevel->nMCVars) || bFound; j++) {
-      pMCVar = plevel->rgpMCVars[j]; /* current Distrib examined */
-      bFound = (pMCVar->hvar == pos->phvar_dat[i]);
-    }
-    if (!bFound)
-      ReportRunTimeError (panal, RE_NOLIKELIHOOD | RE_FATAL,
-                          pos->pszDataNames[i]);
-  }*/
-
-#endif
-
 } /* CheckPrintStatements */
 
 
@@ -442,28 +375,28 @@ void ConvertLists(PLEVEL plevel, char **args)
    set default values for inverse temperatures of tempered MCMC algorithm.
 */
 
-double SetInvTemperatures (PGIBBSDATA pgd) {
+void SetInvTemperatures (PGIBBSDATA pgd) {
 
-  int i, 
+  int i;
 
-  n = pgd->nInvTemperatures = 5;
+  pgd->nInvTemperatures = NTEMP;
 
   /* allocate inverse temperature array */
-  if (!(pgd->rgInvTemperatures = InitdVector (n)))
+  if (!(pgd->rgInvTemperatures = InitdVector (NTEMP)))
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
 
   /* allocate working arrays */
-  if (!(pgd->rgdlnPi = InitdVector (n)) || 
-      !(pgd->rglTemp = InitlVector (n)))
+  if (!(pgd->rgdlnPi = InitdVector (NTEMP)) || 
+      !(pgd->rglTemp = InitlVector (NTEMP)))
     ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
 
   /* initialize the inverse temperatures */
-  pgd->rgInvTemperatures[0] = 0.4;                     /* hot */
+  pgd->rgInvTemperatures[0] = 0.4;                         /* hot */
 
-  for (i = 1; i < n - 1; i++)
-    pgd->rgInvTemperatures[i] = pow(0.8, (n - 1 - i)); /* medium... */
+  for (i = 1; i < NTEMP - 1; i++)
+    pgd->rgInvTemperatures[i] = pow(0.8, (NTEMP - 1 - i)); /* medium... */
 
-  pgd->rgInvTemperatures[n - 1] = 1;                   /* cold */
+  pgd->rgInvTemperatures[NTEMP - 1] = 1;                   /* cold */
 
 } /* SetInvTemperatures */
 
@@ -510,9 +443,12 @@ void DoMarkov (PANALYSIS panal)
   case 3:
     printf ("\nDoing %ld Metropolis within Gibbs tempered simulation",
             nIter);
-    if (pgd->nInvTemperatures == 0) /* inverse temperatures not set by the user: default */
-      SetInvTemperatures (pgd);
     printf ((nIter != 1 ? "s\n" : "\n"));
+
+    /* inverse temperatures not set by the user: default */
+    if (pgd->nInvTemperatures == 0)
+      SetInvTemperatures (pgd);
+
     break;
 
   case 4:
@@ -620,7 +556,8 @@ void DoMarkov (PANALYSIS panal)
 
     if (pgd->nSimTypeFlag == 3)
       ReadRestart2 (pgd->pfileRestart, nThetas, pgd->nInvTemperatures,
-                    pdMCVarVals, pdSum, prgdSumProd, &iter, &pgd->indexT, pgd->rgdlnPi);
+                    pdMCVarVals, pdSum, prgdSumProd, &iter,
+                    &pgd->indexT, pgd->rgdlnPi);
     else
       ReadRestart (pgd->pfileRestart, nThetas, pdMCVarVals, pdSum, prgdSumProd,
                    &iter);
@@ -641,12 +578,24 @@ void DoMarkov (PANALYSIS panal)
     }
 
     /* If component jumps, tempered or stochastic optimization */
-    if ((pgd->nSimTypeFlag == 0) || (pgd->nSimTypeFlag >= 3)) { 
-      /* Set the jumping kernel's SD */
-      TraverseLevels (pLevel0, SetKernel, 1, pdMCVarVals, NULL);
+    if ((pgd->nSimTypeFlag == 0) || (pgd->nSimTypeFlag >= 3)) {
 
-      /* Free the now useless array */
-      free (pdMCVarVals);
+      char szKernelFile[MAX_FILENAMESIZE+7];
+      sprintf(szKernelFile, "%s%s", panal->gd.szGrestart, ".kernel");
+
+      FILE *pfile = fopen (szKernelFile, "r");
+
+      if (pfile) {
+        printf("Reading kernel file %s\n", szKernelFile); 
+        TraverseLevels (pLevel0, ReadKernel, pfile, NULL);
+      }
+      else {
+        /* Set the jumping kernel's SD */
+        TraverseLevels (pLevel0, SetKernel, 1, pdMCVarVals, NULL);
+
+        /* Free the now useless array */
+        free (pdMCVarVals);
+      }
     }
     else { /* vector jumping, initialize prior */
       TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
@@ -749,16 +698,20 @@ void DoMarkov (PANALYSIS panal)
         dLnData = 0.0;
         TraverseLevels1 (pLevel0, SumAllExpts, &dLnData, NULL);
 
-        /* test the temperature */
-        pgd->attemptedTempe++; /* this need to be before the next which uses it */
+        /* update temperature jump counter */
+        pgd->attemptedTempe++; 
 
         /* Robbins-Monro updating of the pseudo prior */
         for (i = 0; i <pgd->nInvTemperatures; i++)
           if (i == pgd->indexT)
-            pgd->rgdlnPi[i] = pgd->rgdlnPi[i] - (pgd->dCZero / (pgd->attemptedTempe + pgd->dNZero));
+            pgd->rgdlnPi[i] = pgd->rgdlnPi[i] - 
+                              (pgd->dCZero / 
+                               (pgd->attemptedTempe + pgd->dNZero));
           else
-            pgd->rgdlnPi[i] = pgd->rgdlnPi[i] + (pgd->dCZero / 
-                              (pgd->nInvTemperatures * (pgd->attemptedTempe + pgd->dNZero)));
+            pgd->rgdlnPi[i] = pgd->rgdlnPi[i] + 
+                              (pgd->dCZero / 
+                               (pgd->nInvTemperatures * 
+                                (pgd->attemptedTempe + pgd->dNZero)));
 
         /* Output indexT and log-densities, eventually */
         if (((iter + 1) % pgd->nPrintFreq == 0) &&
@@ -807,6 +760,25 @@ void DoMarkov (PANALYSIS panal)
 
   } /* while iter */
 
+  /* If component jumps, tempered or stochastic optimization: save kernel */
+  if ((pgd->nSimTypeFlag == 0) || (pgd->nSimTypeFlag >= 3)) {
+
+    char szKernelFile[MAX_FILENAMESIZE+7];
+    sprintf(szKernelFile, "%s%s", panal->gd.szGout, ".kernel");
+
+    FILE *pfile = fopen (szKernelFile, "w");
+    if (!pfile) {
+      printf ("Cannot create kernel saving file '%s'\n", panal->gd.szGdata);
+      exit (0);
+    }
+
+    /* Write out the jumping kernel SDs */
+    TraverseLevels (pLevel0, WriteKernel, pfile, NULL);
+
+    fprintf(pfile, "\n");
+    fclose(pfile);
+    printf("\nWrote kernel SDs to \"%s\"", szKernelFile);
+  }
 
   CloseMarkovFiles (panal);
 
@@ -831,42 +803,59 @@ void DoMarkov (PANALYSIS panal)
 void FindLikeParents (PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
-  long      n, m, l, k;
+  long      k, l, m, n;
   PLEVEL    pPrevLev;
   PMCVAR    pMCVar1, pMCVar2;
+  BOOL      bFound;
 
   /* Set the current level array as we pass through */
   panal->pCurrentLevel[plevel->iDepth] = plevel;
 
-  /* First, this level; Parents do not need to appear before current */
-  for (n = 0; n < plevel->nLikes; n++) {
-    pMCVar1 = plevel->rgpLikes[n]; /* current likelihood */
-    for (m = 0; m < plevel->nMCVars; m++) { /* for all previous distrib */
-      pMCVar2 = plevel->rgpMCVars[m];
-      for (l = 0; l < 4; l++) {
-        if (pMCVar1->hParm[l] == pMCVar2->hvar) {
-          pMCVar1->pMCVParent[l] = pMCVar2;
-        }
-      }
-    }
-  }
+  for (k = 0; k < plevel->nLikes; k++) {
+    pMCVar1 = plevel->rgpLikes[k]; /* current likelihood */
 
-  /* Now, all previous levels */
-  for (n = plevel->iDepth - 1; n >= 0; n--) {
-    pPrevLev = panal->pCurrentLevel[n];
-    for (m = 0; m < plevel->nLikes; m++) {
-      pMCVar1 = plevel->rgpLikes[m]; /* current likelihood */
-      for (l = 0; l < pPrevLev->nMCVars; l++) {
-        pMCVar2 = pPrevLev->rgpMCVars[l];
-        for (k = 0; k < 4; k++) {
-          if (pMCVar1->pMCVParent[k] == NULL &&
-              pMCVar1->hParm[k] == pMCVar2->hvar) {
-            pMCVar1->pMCVParent[k] = pMCVar2;
+    for (l = 0; l < 4; l++) {
+      if (pMCVar1->iParmType[l] == MCVP_PARM) {
+        /* if there is a parent, find it, but parents must appear before
+           current pMCVar */
+        bFound = FALSE;
+
+        /* First, this level */
+        for (m = 0; m < plevel->nMCVars; m++) { /* for all previous distrib */
+          pMCVar2 = plevel->rgpMCVars[m];
+          if (pMCVar1->hParm[l] == pMCVar2->hvar) {
+            pMCVar1->pMCVParent[l] = pMCVar2;
+            bFound = TRUE;
           }
+        } /* for m */
+        
+        /* Now, all previous levels */
+        if (!bFound) {
+          for (n = plevel->iDepth - 1; n >= 0; n--) {
+            pPrevLev = panal->pCurrentLevel[n];
+
+            for (m = 0; m < pPrevLev->nMCVars; m++) {
+              pMCVar2 = pPrevLev->rgpMCVars[m];
+              if (pMCVar1->hParm[l] == pMCVar2->hvar) {
+                pMCVar1->pMCVParent[l] = pMCVar2;
+                bFound = TRUE;
+              }
+            } /* for m */
+          } /* for n */
         }
+
+        if (!bFound) { /* oops, parent not found, error */
+          printf ("\n"
+                  "Error: parent in position %ld of %s must be\n"
+                  "       declared before it when creating\n"
+                  "       sampling dependencies - Exiting.\n\n", 
+                  l, pMCVar1->pszName);
+          exit(0);
+        }
+
       }
-    }
-  }
+    } /* for l */
+  } /* for k */
 
 } /* FindLikeParents */
 
@@ -905,46 +894,63 @@ void FindMCDependents (PLEVEL plevel, char **args)
 void FindMCParents (PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
-  long      n, m, l, k;
+  long      k, l, m, n;
   PLEVEL    pPrevLev;
   PMCVAR    pMCVar1, pMCVar2;
+  BOOL      bFound;
 
   /* Set the current level array as we pass through */
   panal->pCurrentLevel[plevel->iDepth] = plevel;
 
-  /* First, this level; Parents must appear before current in the array */
-  for (n = 0; n < plevel->nMCVars; n++) {
-    pMCVar1 = plevel->rgpMCVars[n]; /* current */
-    for (m = 0; m < n; m++) { /* for all previous */
-      pMCVar2 = plevel->rgpMCVars[m];
-      for (l = 0; l < 4; l++) {
-        if (pMCVar1->hParm[l] == pMCVar2->hvar) {
-          pMCVar1->pMCVParent[l] = pMCVar2;
-        }
-      }
-    }
-  }
+  for (k = 0; k < plevel->nMCVars; k++) {
+    pMCVar1 = plevel->rgpMCVars[k]; /* current distrib */
 
-  /* Now, all previous levels, starting with the one above and going up */
-  for (n = plevel->iDepth-1; n > 0; --n) { /* 27/8/02 FB changed from n >= 0 */
-    pPrevLev = panal->pCurrentLevel[n];
-    for (m = 0; m < plevel->nMCVars; m++) { /* for each var in the level */
-      pMCVar1 = plevel->rgpMCVars[m];
-      /* check each var of the upper level */
-      for (l = 0; l < pPrevLev->nMCVars; l++) {
-        pMCVar2 = pPrevLev->rgpMCVars[l];
-        for (k = 0; k < 4; k++) { /* for the 4 distribution parameters */
-          if (pMCVar1->pMCVParent[k] == NULL &&
-              pMCVar1->hParm[k] == pMCVar2->hvar) {
-            pMCVar1->pMCVParent[k] = pMCVar2;
+    for (l = 0; l < 4; l++) {
+      if (pMCVar1->iParmType[l] == MCVP_PARM) {
+        /* if there is a parent, find it, but parents must appear before
+           current pMCVar */
+        bFound = FALSE;
+
+        /* First, this level */
+        for (m = 0; m < k; m++) { /* for all previous distrib */
+          pMCVar2 = plevel->rgpMCVars[m];
+          if (pMCVar1->hParm[l] == pMCVar2->hvar) {
+            pMCVar1->pMCVParent[l] = pMCVar2;
+            bFound = TRUE;
           }
+        } /* for m */
+        
+        /* Now, all previous levels */
+        if (!bFound) {
+          for (n = plevel->iDepth - 1; n >= 0; n--) {
+            pPrevLev = panal->pCurrentLevel[n];
+
+            for (m = 0; m < pPrevLev->nMCVars; m++) {
+              pMCVar2 = pPrevLev->rgpMCVars[m];
+              if (pMCVar1->hParm[l] == pMCVar2->hvar) {
+                pMCVar1->pMCVParent[l] = pMCVar2;
+                bFound = TRUE;
+              }
+            } /* for m */
+          } /* for n */
         }
+
+        if (!bFound) { /* oops, parent not found, error */
+          printf ("\n"
+                  "Error: parent in position %ld of %s must be\n"
+                  "       declared before it when creating\n"
+                  "       sampling dependencies - Exiting.\n\n", 
+                  l, pMCVar1->pszName);
+          exit(0);
+        }
+
       }
-    }
-  }
+    } /* for l */
+  } /* for k */
 
 } /* FindMCParents */
 
+
 /* Function -------------------------------------------------------------------
    GetNumberOfMCVars
 
@@ -1698,8 +1704,7 @@ void PrintDeps (PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    ReadRestart
 
-   initialize the population and individual parameters by reading them in the
-   restart file.
+   initialize the parameters by reading them in the restart file.
 */
 
 void ReadRestart (FILE *pfileRestart, long nThetas,
@@ -1762,8 +1767,8 @@ void ReadRestart (FILE *pfileRestart, long nThetas,
 /* Function -------------------------------------------------------------------
    ReadRestart2
 
-   initialize the population and individual parameters and temperature stuff
-   (pseudoprior, indexT) by reading them in the restart file.
+   initialize parameters and temperature stuff (pseudoprior, indexT) by reading
+   them in the restart file.
    */
 
 void ReadRestart2 (FILE *pfileRestart, long nThetas, int nInvTemperatures,
@@ -1938,12 +1943,11 @@ int RunExpt (PLEVEL plevel, char **args)
 
 long SampleTemperature (PGIBBSDATA pgd, double dLnPrior, double dLnData)
 {
-  long   indexT = pgd->indexT;
-  long   indexT_new;
-  double pseudo, pseudonew;
+  int    indexT = pgd->indexT;
+  int    indexT_new;
 
   /* Propose a new inverse temperature */
-  if (indexT == 0) indexT_new = indexT + 1;
+  if (indexT == 0) indexT_new = 1;
   else {
     if (indexT == pgd->nInvTemperatures - 1) indexT_new = indexT - 1;
     else {
@@ -1954,11 +1958,9 @@ long SampleTemperature (PGIBBSDATA pgd, double dLnPrior, double dLnData)
 
   /* Test the temperature */
 
-  pseudo = pgd->rgdlnPi[indexT];
-  pseudonew = pgd->rgdlnPi[indexT_new];
-
   if (Test2 (pgd, indexT, indexT_new, dLnPrior, dLnData, 
-             pgd->attemptedTempe, pseudo, pseudonew)) {   
+             pgd->attemptedTempe, pgd->rgdlnPi[indexT], 
+             pgd->rgdlnPi[indexT_new])) {   
     /* jump */
     return (indexT_new);
   }
@@ -2275,8 +2277,13 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
     /* save the current kernelSD */
     old_dKernelSD =  pMCVar->dKernelSD;
 
-    /* update the kernelSD according to the temperature */
-    pMCVar->dKernelSD = pMCVar->dKernelSD / pgd->rgInvTemperatures[*pindexT];
+    /* update the kernelSD according to the temperature, the formula
+       comes from that of the variance of the powered normal */
+    pMCVar->dKernelSD = pow(2 * PI, 
+                            0.5 * (1 - pgd->rgInvTemperatures[*pindexT])) *
+                        pow(pMCVar->dKernelSD, 
+                            3 - pgd->rgInvTemperatures[*pindexT]) /
+                        pow(pgd->rgInvTemperatures[*pindexT], 1.5);
 
     /* check that kernel SD does not increase wildly */
     if (pMCVar->dKernelSD > pMCVar->dMaxKernelSD)
@@ -2304,7 +2311,7 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
     }
 
     /* Test the results and act accordingly */
-    if (!Test1 (pgd, pMCVar->bExptIsDep,  dLnPrior, dLnPriorNew, dLnLike,
+    if (!Test1 (pgd, pMCVar->bExptIsDep, dLnPrior, dLnPriorNew, dLnLike,
                 dLnLikeNew, dLnData, dLnDataNew, *pindexT)) {
       pMCVar->dVal = dTheta;
        if(pMCVar->bExptIsDep)
@@ -2576,6 +2583,57 @@ void SetKernel (PLEVEL plevel, char **args)
 } /* SetKernel */
 
 
+/* Function -------------------------------------------------------------------
+   WriteKernel
+
+   Write values of the MCMC jumping kernel
+*/
+
+void WriteKernel (PLEVEL plevel, char **args)
+{
+  FILE   *pfile = (FILE *) args[0];
+  long   n;
+  PMCVAR pMCVar;
+
+  for (n = 0; n < plevel->nMCVars; n++) {
+    if ( !(plevel->rgpMCVars[n]->bIsFixed)) {
+      pMCVar = plevel->rgpMCVars[n];
+      fprintf(pfile, "%lg\t", pMCVar->dKernelSD);
+    }
+  }
+
+} /* WriteKernel */
+
+
+/* Function -------------------------------------------------------------------
+   ReadKernel
+
+   Read values of the MCMC jumping kernel
+*/
+
+void ReadKernel (PLEVEL plevel, char **args)
+{
+  FILE   *pfile = (FILE *) args[0];
+  long   n;
+  PMCVAR pMCVar;
+
+  for (n = 0; n < plevel->nMCVars; n++) {
+    if ( !(plevel->rgpMCVars[n]->bIsFixed)) {
+      pMCVar = plevel->rgpMCVars[n];
+
+      /* set the maximum kernel SD value */
+      if (pMCVar->iType == MCV_UNIFORM || pMCVar->iType == MCV_LOGUNIFORM )
+        pMCVar->dMaxKernelSD = *(pMCVar->pdParm[1]) / 6.0 - *(pMCVar->pdParm[0]) / 6.0;
+      else
+        pMCVar->dMaxKernelSD = *(pMCVar->pdParm[3]) / 6.0 - *(pMCVar->pdParm[2]) / 6.0;
+
+      fscanf(pfile, "%lg", &(pMCVar->dKernelSD));
+    }
+  }
+
+} /* ReadKernel */
+
+
 /* Function -------------------------------------------------------------------
    SetModelVars
 
@@ -2765,22 +2823,6 @@ int SumAllExpts (PLEVEL plevel, char **args)
 } /* SumAllExpts */
 
 
-/* Function -----------------------------------------------------------------
-   Temper_probabilities
-
-   Set the temperature transition probabilities.
-*/
-
-double Temper_probabilities (int n, long i, long j)
-{
-  if ((i == 0 && j == 1) || (i == n - 1 && j == n)) 
-    return (1.0);
-  else 
-    return (0.5);
-
-} /* Temper_probabilities */
-
-
 /* Function -------------------------------------------------------------------
    Test
 
@@ -2820,11 +2862,12 @@ BOOL Test (PGIBBSDATA pgd, BOOL bExptIsDep,
 /* Function -------------------------------------------------------------------
    Test1
 
-   Test prior, likelihood against random number between 0 and 1 according to 
+   Test prior, likelihood against a random number between 0 and 1 according to 
    the temperature
 */
 
-BOOL Test1 (PGIBBSDATA pgd, BOOL bExptIsDep, double dLnPrior, double dLnPriorNew,
+BOOL Test1 (PGIBBSDATA pgd, BOOL bExptIsDep,
+            double dLnPrior, double dLnPriorNew,
             double dLnLike, double dLnLikeNew,
             double dLnData, double dLnDataNew, long indexT)
 {
@@ -2849,29 +2892,32 @@ BOOL Test1 (PGIBBSDATA pgd, BOOL bExptIsDep, double dLnPrior, double dLnPriorNew
 /* Function -------------------------------------------------------------------
    Test2
 
-   Test temperature against random number between 0 and 1
+   Test temperature against a random number between 0 and 1
 */
 
 BOOL Test2 (PGIBBSDATA pgd, long indexT, long indexT_new, double dLnPrior, 
-            double dLnData, long attemptedTempe, double pseudo, double pseudonew)
+            double dLnData, long attemptedTempe,
+            double pseudo, double pseudonew)
 {
   double dPjump = 0;
+  #define MINUSLN2  -0.6931471805599452862268
 
   if (dLnPrior + dLnData == NULL_SUPPORT)
     return FALSE;
 
-  dPjump = (pgd->rgInvTemperatures[indexT_new] - pgd->rgInvTemperatures[indexT]) * 
-           (dLnPrior + dLnData) +
-           pseudonew - pseudo + 
-           log(Temper_probabilities(pgd->nInvTemperatures, indexT_new, indexT)) -
-           log(Temper_probabilities(pgd->nInvTemperatures, indexT, indexT_new));
+  dPjump = (pgd->rgInvTemperatures[indexT_new] -
+            pgd->rgInvTemperatures[indexT]) * (dLnPrior + dLnData) +
+           pseudonew - pseudo +
+           ((indexT_new == 0) || (indexT_new == pgd->nInvTemperatures - 1) ? 
+            0 : MINUSLN2) -
+           ((indexT     == 0) || (indexT     == pgd->nInvTemperatures - 1) ? 
+            0 : MINUSLN2);
 
   return ((BOOL) (log(Randoms()) <= dPjump));
 
 } /* Test2 */
 
 
-
 /* Function -------------------------------------------------------------------
    TraverseLevels (recursive)
 
