@@ -350,12 +350,12 @@ void UpdateDefaultInput (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 {
   double dT; /* New time to try */
-  int i;
+  int i, j, stateID;
 
   if (vbModelReinitd) {         /* if model has been re-initialized */
     ScaleModel(pdTime);         /* scale the model parms */
-    FixupDependentInputs ();    /* fixup input fields dependent on vars */
-    GetStartPeriods (pdTime);   /* initialize the current period */
+    FixupDependentInputs();     /* fixup input fields dependent on vars */
+    GetStartPeriods(pdTime);    /* initialize the current period */
   }
 
   dT = *pdNextTransTime = DBL_MAX;
@@ -369,6 +369,25 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
         UpdateNDoses (&vrgInputs[i], &dT, pdTime);
         break;
 
+      case IFN_EVENTS:
+        if (vrgInputs[i].bOn)
+          vrgInputs[i].iDoseCur++;
+
+        UpdateSpikes (&vrgInputs[i], &dT, pdTime);
+
+        j = vrgInputs[i].iDoseCur;
+        stateID = HINDEX(vrgInputs[i].target_state);
+        if (vrgInputs[i].bOn && (j < vrgInputs[i].nDoses))
+          vrgModelVars[stateID] = 
+            (vrgInputs[i].rgOper[j] == KM_REPLACE ? 
+              vrgInputs[i].rgMags[j] : 
+             vrgInputs[i].rgOper[j] == KM_ADD     ? 
+              vrgModelVars[stateID] + vrgInputs[i].rgMags[j] : 
+	     vrgModelVars[stateID] * vrgInputs[i].rgMags[j]);
+
+        break;
+
+
       case IFN_SPIKES:
         /* FB 9/9/97: make it advance only if this input spike is on.
            At the start it is off. The on/off switch is only used here,
