@@ -2,7 +2,7 @@
 
    Originally written by Don Maszle
 
-   Copyright (c) 1993-2011 Free Software Foundation, Inc.
+   Copyright (c) 1993-2015 Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -50,8 +50,8 @@
 */
 
 /* Version and copyright */
-#define VSZ_VERSION "v5.5.0"
-#define VSZ_COPYRIGHT "Copyright (c) 1993-2013 Free Software Foundation, Inc."
+#define VSZ_VERSION "v5.6.0"
+#define VSZ_COPYRIGHT "Copyright (c) 1993-2015 Free Software Foundation, Inc."
 
 /* These are potential array size problems.
    Other maximum sizes are MAX_EQN and MAX_LEX in lex.h */
@@ -129,6 +129,10 @@
 #define KM_FORWARD         700
 #define KM_BACKWARD        701
 
+#define KM_REPLACE         710
+#define KM_ADD             711
+#define KM_MULTIPLY        712
+
 /* Context bit flags */
 
 #define CN_END             0x0000    /* No context */
@@ -236,7 +240,7 @@ typedef struct tagINTSPEC {
   long    itol;           /* type of error checking */
   double  dRtol;          /* relative error tolerance */
   double  dAtol;          /* aboslute error tolerance */
-  long    iMf;            /* 0 = nonstiff, 1 = stiff */
+  long    iMf;            /* 0: nonstiff; 1: stiff; 2: stiff, jac supplied */
   long    iDSFlag;        /* lsodes return flag */
   long    liw;            /* length of iwork array */
   long    lrw;            /* length of rwork array */
@@ -408,7 +412,9 @@ typedef struct tagMODELINFO {
 typedef struct tagEXPERIMENT {
   int iExp;                 /* Number of this experiment */
 
-  double dT0;               /* Time limits */
+  double dT0;               /* Initial time */
+  HANDLE hT0;               /* Handle to initial time */
+
   double dTfinal;
   double dTime;             /* Current Time */
 
